# -*- coding: utf-8; -*-
################################################################################
#
#  WuttaPOS -- Pythonic Point of Sale System
#  Copyright © 2023 Lance Edgar
#
#  This file is part of WuttaPOS.
#
#  WuttaPOS is free software: you can redistribute it and/or modify it under the
#  terms of the GNU General Public License as published by the Free Software
#  Foundation, either version 3 of the License, or (at your option) any later
#  version.
#
#  WuttaPOS is distributed in the hope that it will be useful, but WITHOUT ANY
#  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
#  FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
#  details.
#
#  You should have received a copy of the GNU General Public License along with
#  WuttaPOS.  If not, see <http://www.gnu.org/licenses/>.
#
################################################################################
"""
WuttaPOS - login view
"""

import flet as ft

from .base import WuttaView
from wuttapos.controls.keyboard import WuttaKeyboard


class LoginView(WuttaView):
    """
    Main POS view for WuttaPOS
    """

    def __init__(self, *args, **kwargs):

        # TODO: maybe support setting this to False?  for now that's not 100% supported
        self.show_username = kwargs.pop('show_username', True)

        super().__init__(*args, **kwargs)

        # track which login input has focus
        self.focused = None

    def build_controls(self):
        title = self.app.get_title()
        controls = [
            ft.Row(
                [ft.Text(value=f"Welcome to {title}", weight=ft.FontWeight.BOLD, size=28)],
                alignment=ft.MainAxisAlignment.CENTER,
            ),
            ft.Row(),
            ft.Row(),
            ft.Row(),
        ]

        form_fields = []

        self.password = ft.TextField(label="Password", width=200, password=True,
                                     on_submit=self.password_submit,
                                     on_focus=self.password_focus,
                                     autofocus=not self.show_username)
        self.focused = self.password

        if self.show_username:
            self.username = ft.TextField(label="Login", width=200,
                                         on_submit=self.username_submit,
                                         on_focus=self.username_focus,
                                         autofocus=True)
            form_fields.append(self.username)
            self.focused = self.username

        form_fields.append(self.password)

        controls.extend([
            ft.Row(
                controls=form_fields + [
                    ft.Container(content=ft.Text("Login", size=20,
                                                 weight=ft.FontWeight.BOLD),
                                 height=60,
                                 width=60 * 2.5,
                                 alignment=ft.alignment.center,
                                 border=ft.border.all(1, 'black'),
                                 border_radius=ft.border_radius.all(5),
                                 bgcolor='blue',
                                 on_click=self.attempt_login),

                    ft.Container(content=ft.Text("Clear", size=20,
                                                 weight=ft.FontWeight.BOLD),
                                 height=60,
                                 width=60 * 2.5,
                                 alignment=ft.alignment.center,
                                 border=ft.border.all(1, 'black'),
                                 border_radius=ft.border_radius.all(5),
                                 on_click=self.clear_login),
                ],
                alignment=ft.MainAxisAlignment.CENTER,
            ),
            ft.Row(),
            ft.Row(),
            ft.Row(),
            WuttaKeyboard(self.config, on_keypress=self.keypress,
                          on_long_backspace=self.long_backspace),
        ])

        return [
            self.build_header(),
            ft.Column(controls=controls,
                      expand=True,
                      alignment=ft.MainAxisAlignment.CENTER),
        ]

    def keypress(self, key):
        assert self.focused
        if key == '⏎':
            if self.focused is self.username:
                self.username_submit()
            else:
                self.password_submit()
        elif key == '⌫':
            self.focused.value = self.focused.value[:-1]
        else:
            self.focused.value += key

        self.focused.focus()
        self.update()

    def long_backspace(self):
        assert self.focused
        self.focused.value = ''
        self.focused.focus()
        self.update()

    def username_focus(self, e):
        self.focused = self.username

    def username_submit(self, e=None):
        if self.username.value:
            self.password.focus()
        else:
            self.username.focus()

    def password_focus(self, e):
        self.focused = self.password

    def password_submit(self, e=None):
        if self.password.value:
            self.attempt_login()
        else:
            self.password.focus()

    def clear_login(self, e):
        if self.show_username:
            self.username.value = ""
        self.password.value = ""
        if self.show_username:
            self.username.focus()
        else:
            self.password.focus()
        self.page.update()

    def attempt_login(self, e=None):
        if self.show_username and not self.username.value:
            self.username.focus()
            return
        if not self.password.value:
            self.password.focus()
            return

        session = self.app.make_session()
        auth = self.app.get_auth_handler()
        user = auth.authenticate_user(session, self.username.value, self.password.value)
        user_display = str(user) if user else None
        session.close()

        if user:
            # handle success
            self.page.session.set('user_uuid', user.uuid)
            self.page.session.set('user_display', user_display)

            # TODO: hacky but works for now
            if not self.config.production():
                self.page.client_storage.set('user_uuid', user.uuid)

            self.page.go('/pos')

        else:
            # handle failure
            self.page.snack_bar = ft.SnackBar(ft.Text("Login failed!",
                                                      color='black',
                                                      weight=ft.FontWeight.BOLD),
                                              bgcolor='yellow',
                                              duration=1500)
            self.page.snack_bar.open = True
            self.password.focus()
            self.page.update()
